@@ -3,6 +3,7 @@ use arrow::array::BinaryArray;
 use arrow::array::BooleanArray;
 use arrow::array::Date32Array;
 use arrow::array::Date64Array;
+use arrow::array::Decimal128Array;
 use arrow::array::FixedSizeBinaryArray;
 use arrow::array::Float16Array;
 use arrow::array::Float32Array;
@@ -26,6 +27,7 @@ use arrow::record_batch::RecordBatch;
 use std::borrow::Cow;
 use std::fmt::Display;
 use std::time::Duration as StdDuration;
+use tiberius::numeric::Numeric;
 use tiberius::time::time::Date;
 use tiberius::time::time::PrimitiveDateTime;
 use tiberius::time::time::Time;
@@ -86,7 +88,15 @@ pub(crate) fn get_token_rows<'a>(
 
     let rows = batch.num_rows();
     let mut token_rows: Vec<TokenRow> = vec![TokenRow::new(); rows.try_into()?];
-    for colname in cols {
+
+    let batchcols = batch
+        .schema()
+        .fields()
+        .iter()
+        .map(|x| x.name().to_string())
+        .collect::<Vec<String>>();
+    let colsnames = if cols.len() == 0 { &batchcols } else { cols };
+    for colname in colsnames {
         let mightcol = batch.column_by_name(colname);
         if let None = mightcol {
             for rowindex in 0..rows {
@@ -418,6 +428,17 @@ pub(crate) fn get_token_rows<'a>(
                     rowindex += 1;
                 }
             }
+            arrow::datatypes::DataType::Decimal128(_, s) => {
+                let ba = col.as_any().downcast_ref::<Decimal128Array>().unwrap();
+                let scale: u8 = s.clone().try_into()?;
+                let mut rowindex = 0;
+                for val in ba.iter() {
+                    token_rows[rowindex].push(ColumnData::Numeric(
+                        val.map(|x| Numeric::new_with_scale(x, scale)),
+                    ));
+                    rowindex += 1;
+                }
+            }
             dt => return Err(Box::new(NotSupportedError { dtype: dt.clone() })), //other => panic!("Not supported {:?}", other),
         }
     }
